package cs636.pizza.dao;
// Example JUnit5 test 
// Important: use this import for Test, not the JUnit4 import for Test
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.JdbcTest;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;

import java.sql.Connection;
import java.sql.SQLException;

//@RunWith(SpringRunner.class)
// Junit5--
@ExtendWith(SpringExtension.class)
// Needed to handle DataSource config
@JdbcTest
// This sets up full set of beans, not all needed
//@ContextConfiguration(classes = {AppConfig.class})
//to be minimalistic, configure only the needed beans, avoid using AppConfig
@ContextConfiguration(classes= {DbDAO.class, MenuDAO.class})
// use application-test.properties in src/main/resources instead of application.properties
@ActiveProfiles("test")

//import cs636.pizza.config.StandAloneDataSourceFactory;
public class MenuDAOTest {
	@Autowired
	private DbDAO db;
	@Autowired
	private MenuDAO menuDAO;
	private Connection connection;
	
    // JUnit 4 @After is now Junit 5 @AfterEach
	@AfterEach
	public void tearDown() throws Exception {
		// This executes even after an exception
		// so we need to rollback here in case of exception
		// (If the transaction was successful, it's already
		// committed, and this won't hurt.)
		db.rollbackAfterException(connection);
	}
	
	@Test
	public void testCreateTopping() throws SQLException
	{
		connection = db.startTransaction();
		menuDAO.createMenuTopping(connection, "anchovies");	
		int count = menuDAO.findMenuToppings(connection).size();
		Assertions.assertTrue( count == 1, () -> "first topping not created");
		db.commitTransaction(connection);
	}
	
	@Test
	public void testCreateSize() throws SQLException
	{
		connection = db.startTransaction();
		menuDAO.createMenuSize(connection, "huge");	
		int count = menuDAO.findMenuSizes(connection).size();
		Assertions.assertTrue( count == 1, () -> "first topping not created");
		db.commitTransaction(connection);
	}

}
